/*
 * Copyright 2018 by Kappich Systemberatung Aachen
 *
 * This file is part of de.kappich.pat.testumg.
 *
 * de.kappich.pat.testumg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.kappich.pat.testumg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.kappich.pat.testumg.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.kappich.pat.testumg.util;

import de.bsvrz.dav.daf.main.config.ObjectSet;

import javax.tools.StandardJavaFileManager;
import java.io.IOException;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.FileAttribute;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 *
 *
 * @author Kappich Systemberatung
 */
public class TempDirectoryCreator {

	private static final Thread _cleaner = new Thread() {
		@Override
		public void run() {
			for(Path temporaryDirectory : _temporaryDirectories) {
				try {
					deletePath(temporaryDirectory);
				}
				catch(IOException e) {
					throw new AssertionError(e);
				}
			}
		}
	};
	private static final List<Path> _temporaryDirectories = new CopyOnWriteArrayList<>();

	static {
		Runtime.getRuntime().addShutdownHook(_cleaner);
	}

	public static Path createTemporaryDirectory() {
		Path path = createTemporaryDirectoryInternal();
		_temporaryDirectories.add(path);
		return path;
	}

	private static Path createTemporaryDirectoryInternal() {
		try {
			String basePath = System.getProperty("testbasepath", null);
			if(basePath == null)
				return Files.createTempDirectory("test");
			else {
				return Files.createTempDirectory(Paths.get(basePath), "test");
			}
		}
		catch(IOException e) {
			throw new AssertionError(e);
		}
	}


	public static void deletePath(final Path path) throws IOException {
		if(!Files.exists(path)) return;
		Files.walkFileTree(path, new SimpleFileVisitor<Path>()
		{
			@Override
			public FileVisitResult visitFile(Path file, BasicFileAttributes attrs)
					throws IOException
			{
				Files.delete(file);
				return FileVisitResult.CONTINUE;
			}

			@Override
			public FileVisitResult visitFileFailed(Path file, IOException exc) throws IOException
			{
				return FileVisitResult.CONTINUE;
			}

			@Override
			public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException
			{
				Files.delete(dir);
				return FileVisitResult.CONTINUE;
			}
		});
	}

}
